@@ -54,6 +54,7 @@ use alloc::{
     vec::Vec,
 };
 use core::ops::{Deref, DerefMut};
+use ritelinked::LinkedHashMap;
 
 mod base;
 mod error;
@@ -64,18 +65,23 @@ macro_rules! tag_prefix {
     };
 }
 
-macro_rules! match_or {
-    ($e:expr, $then:expr) => {
-        match $e {
+macro_rules! or {
+    ($e1:expr; $e2:expr) => {
+        match $e1 {
             Ok(v) => Ok(v),
-            Err(PError::Mismatch) => $then,
+            Err(PError::Mismatch) => $e2,
             Err(e) => Err(e),
         }
     };
+    ($e1:expr; $($e2:expr);+) => {
+        or!($e1; or!($($e2);+))
+    };
 }
 
 pub(crate) use tag_prefix;
 
+/// The type of anchor recorder in the [`Loader`].
+pub type Anchors<R> = LinkedHashMap<String, Node<R>>;
 /// The default prefix of the YAML sub tag.
 pub const DEFAULT_PREFIX: &str = tag_prefix!();
 
@@ -108,8 +114,8 @@ pub const DEFAULT_PREFIX: &str = tag_prefix!();
 pub struct Loader<'a, R: Repr> {
     /// Parser base.
     pub parser: Parser<'a>,
-    /// A visitor of anchors.
-    pub anchors: Anchor<R>,
+    keep_anchors: bool,
+    anchors: Anchors<R>,
 }
 
 impl<'a, R: Repr> Loader<'a, R> {
@@ -117,7 +123,8 @@ impl<'a, R: Repr> Loader<'a, R> {
     pub fn new(doc: &'a [u8]) -> Self {
         Self {
             parser: Parser::new(doc),
-            anchors: Anchor::new(),
+            keep_anchors: false,
+            anchors: Anchors::new(),
         }
     }
 }
@@ -138,8 +145,24 @@ impl<'a, R: Repr> Loader<'a, R> {
 ///
 /// The `inner` parameter presents that the expression is in a **flow** expression.
 impl<R: Repr> Loader<'_, R> {
+    /// Keep the anchor insertion.
+    ///
+    /// This will make [`Yaml::Alias`] have a placeholder
+    /// and adding anchor information in the [`Node`].
+    pub fn keep_anchors(self, keep_anchors: bool) -> Self {
+        Self {
+            keep_anchors,
+            ..self
+        }
+    }
+
+    /// Consume this loader and return the recorded anchors.
+    pub fn get_anchors(self) -> Anchors<R> {
+        self.anchors
+    }
+
     /// YAML entry point, return entire doc if exist.
-    pub fn parse(&mut self) -> PResult<Seq<R>> {
+    pub fn parse(&mut self) -> PResult<Vec<Node<R>>> {
         loop {
             match self.context(Parser::directive) {
                 Ok(()) => (),
@@ -195,15 +218,16 @@ impl<R: Repr> Loader<'_, R> {
 
     /// Match scalar.
     pub fn scalar(&mut self, level: usize, nest: bool, inner: bool) -> PResult<Node<R>> {
-        self.scalar_inner(|p| {
+        self.scalar_node(|p| {
             if let Ok(s) = p.string_literal(level) {
-                Ok(Yaml::Str(s))
+                Ok(R::repr(Yaml::Str(s)))
             } else if let Ok(s) = p.string_folded(level) {
-                Ok(Yaml::Str(s))
+                Ok(R::repr(Yaml::Str(s)))
             } else {
-                match_or!(
-                    p.seq(level, nest),
-                    match_or!(p.map(level, nest, inner), p.scalar_term(level, inner))
+                or!(
+                    p.seq(level, nest);
+                    p.map(level, nest, inner);
+                    p.scalar_term(level, inner)
                 )
             }
         })
@@ -211,12 +235,12 @@ impl<R: Repr> Loader<'_, R> {
 
     /// Match flow scalar.
     pub fn scalar_flow(&mut self, level: usize, inner: bool) -> PResult<Node<R>> {
-        self.scalar_inner(|p| p.scalar_term(level, inner))
+        self.scalar_node(|p| p.scalar_term(level, inner))
     }
 
-    fn scalar_inner<F>(&mut self, f: F) -> PResult<Node<R>>
+    fn scalar_node<F>(&mut self, f: F) -> PResult<Node<R>>
     where
-        F: Fn(&mut Self) -> PResult<Yaml<R>>,
+        F: FnOnce(&mut Self) -> PResult<R::Ty>,
     {
         let anchor = self.anchor().unwrap_or_default();
         if !anchor.is_empty() {
@@ -231,7 +255,7 @@ impl<R: Repr> Loader<'_, R> {
         let pos = self.indicator();
         let yaml = f(self)?;
         self.forward();
-        let node = Node::new(yaml, pos, &tag, &anchor);
+        let node = Node::new_repr(yaml, pos, &tag);
         if !anchor.is_empty() && self.anchors.insert(anchor, node.clone()).is_some() {
             self.err("duplicated anchor definition")
         } else {
@@ -240,21 +264,29 @@ impl<R: Repr> Loader<'_, R> {
     }
 
     /// Match flow scalar terminal.
-    pub fn scalar_term(&mut self, level: usize, inner: bool) -> PResult<Yaml<R>> {
+    pub fn scalar_term(&mut self, level: usize, inner: bool) -> PResult<R::Ty> {
         let yaml = if let Ok(s) = self.float() {
-            Yaml::Float(s)
+            R::repr(Yaml::Float(s))
         } else if let Ok(s) = self.sci_float() {
-            Yaml::Float(s)
+            R::repr(Yaml::Float(s))
         } else if let Ok(s) = self.int() {
-            Yaml::Int(s)
+            R::repr(Yaml::Int(s))
         } else if let Ok(s) = self.anchor_use() {
-            Yaml::Alias(s)
+            if let Some(node) = self.anchors.get(&s) {
+                if self.keep_anchors {
+                    R::repr(Yaml::Alias(s))
+                } else {
+                    node.clone_yaml()
+                }
+            } else {
+                return self.err("anchor referenced before definition");
+            }
         } else if let Ok(s) = self.string_quoted(b'\'', b"''") {
-            Yaml::Str(s)
+            R::repr(Yaml::Str(s))
         } else if let Ok(s) = self.string_quoted(b'"', b"\\\"") {
-            Yaml::Str(Parser::escape(&s))
+            R::repr(Yaml::Str(Parser::escape(&s)))
         } else if let Ok(s) = self.string_plain(level, inner) {
-            match s.as_str() {
+            R::repr(match s.as_str() {
                 "~" | "null" | "Null" | "NULL" => Yaml::Null,
                 "true" | "True" | "TRUE" => Yaml::Bool(true),
                 "false" | "False" | "FALSE" => Yaml::Bool(false),
@@ -262,18 +294,19 @@ impl<R: Repr> Loader<'_, R> {
                 ".inf" | ".Inf" | ".INF" => Yaml::Float("inf".to_string()),
                 "-.inf" | "-.Inf" | "-.INF" => Yaml::Float("-inf".to_string()),
                 _ => Yaml::Str(s),
-            }
+            })
         } else {
-            match_or!(
-                self.seq_flow(level),
-                match_or!(self.map_flow(level), Ok(Yaml::Null))
+            or!(
+                self.seq_flow(level);
+                self.map_flow(level);
+                Ok(R::repr(Yaml::Null))
             )?
         };
         Ok(yaml)
     }
 
     /// Match flow sequence.
-    pub fn seq_flow(&mut self, level: usize) -> PResult<Yaml<R>> {
+    pub fn seq_flow(&mut self, level: usize) -> PResult<R::Ty> {
         self.sym(b'[')?;
         let mut v = vec![];
         loop {
@@ -283,8 +316,8 @@ impl<R: Repr> Loader<'_, R> {
                 break;
             }
             self.forward();
-            v.push(match_or!(
-                self.scalar(level + 1, false, true),
+            v.push(or!(
+                self.scalar(level + 1, false, true);
                 self.err("flow sequence item")
             )?);
             self.inv(TakeOpt::More(0))?;
@@ -295,11 +328,11 @@ impl<R: Repr> Loader<'_, R> {
             }
         }
         self.forward();
-        Ok(v.into_iter().collect())
+        Ok(R::repr(v.into_iter().collect()))
     }
 
     /// Match flow map.
-    pub fn map_flow(&mut self, level: usize) -> PResult<Yaml<R>> {
+    pub fn map_flow(&mut self, level: usize) -> PResult<R::Ty> {
         self.sym(b'{')?;
         let mut m = vec![];
         loop {
@@ -311,8 +344,8 @@ impl<R: Repr> Loader<'_, R> {
             self.forward();
             let k = if self.complex_mapping().is_ok() {
                 self.forward();
-                let k = match_or!(
-                    self.scalar(level + 1, false, true),
+                let k = or!(
+                    self.scalar(level + 1, false, true);
                     self.err("flow map key")
                 )?;
                 if self.gap(true).is_ok() {
@@ -320,14 +353,14 @@ impl<R: Repr> Loader<'_, R> {
                 }
                 k
             } else {
-                match_or!(self.scalar_flow(level + 1, true), self.err("flow map key"))?
+                or!(self.scalar_flow(level + 1, true); self.err("flow map key"))?
             };
             if self.sym(b':').is_err() || self.bound().is_err() {
                 return self.err("flow map splitter");
             }
             self.forward();
-            let v = match_or!(
-                self.scalar(level + 1, false, true),
+            let v = or!(
+                self.scalar(level + 1, false, true);
                 self.err("flow map value")
             )?;
             m.push((k, v));
@@ -338,11 +371,11 @@ impl<R: Repr> Loader<'_, R> {
             }
         }
         self.forward();
-        Ok(m.into_iter().collect())
+        Ok(R::repr(m.into_iter().collect()))
     }
 
     /// Match sequence.
-    pub fn seq(&mut self, level: usize, nest: bool) -> PResult<Yaml<R>> {
+    pub fn seq(&mut self, level: usize, nest: bool) -> PResult<R::Ty> {
         let mut v = vec![];
         loop {
             self.forward();
@@ -375,18 +408,18 @@ impl<R: Repr> Loader<'_, R> {
                 }
             }
             self.forward();
-            v.push(match_or!(
-                self.scalar(if downgrade { level } else { level + 1 }, false, false),
+            v.push(or!(
+                self.scalar(if downgrade { level } else { level + 1 }, false, false);
                 self.err("sequence item")
             )?);
         }
         // Keep last wrapping
         self.backward();
-        Ok(v.into_iter().collect())
+        Ok(R::repr(v.into_iter().collect()))
     }
 
     /// Match map.
-    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> PResult<Yaml<R>> {
+    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> PResult<R::Ty> {
         let mut m = vec![];
         loop {
             self.forward();
@@ -402,7 +435,7 @@ impl<R: Repr> Loader<'_, R> {
                 self.forward();
                 let k = if self.complex_mapping().is_ok() {
                     self.forward();
-                    let k = match_or!(self.scalar(level + 1, true, inner), self.err("map key"))?;
+                    let k = or!(self.scalar(level + 1, true, inner); self.err("map key"))?;
                     if self.gap(true).is_ok() {
                         self.ind(level)?;
                     }
@@ -412,7 +445,7 @@ impl<R: Repr> Loader<'_, R> {
                 };
                 if self.sym(b':').is_err() || self.bound().is_err() {
                     // Return key
-                    return Ok(k.yaml().clone());
+                    return Ok(k.clone_yaml());
                 }
                 k
             } else {
@@ -425,13 +458,13 @@ impl<R: Repr> Loader<'_, R> {
                 self.forward();
                 let k = if self.complex_mapping().is_ok() {
                     self.forward();
-                    let k = match_or!(self.scalar(level + 1, true, inner), self.err("map key"))?;
+                    let k = or!(self.scalar(level + 1, true, inner); self.err("map key"))?;
                     if self.gap(true).is_ok() {
                         self.ind(level)?;
                     }
                     k
                 } else {
-                    match_or!(self.scalar_flow(level + 1, inner), self.err("map key"))?
+                    or!(self.scalar_flow(level + 1, inner); self.err("map key"))?
                 };
                 if self.sym(b':').is_err() || self.bound().is_err() {
                     return self.err("map splitter");
@@ -439,12 +472,12 @@ impl<R: Repr> Loader<'_, R> {
                 k
             };
             self.forward();
-            let v = match_or!(self.scalar(level + 1, true, false), self.err("map value"))?;
+            let v = or!(self.scalar(level + 1, true, false); self.err("map value"))?;
             m.push((k, v));
         }
         // Keep last wrapping
         self.backward();
-        Ok(m.into_iter().collect())
+        Ok(R::repr(m.into_iter().collect()))
     }
 }
 
@@ -463,7 +496,7 @@ impl<R: Repr> DerefMut for Loader<'_, R> {
 }
 
 /// Parse YAML document into [`alloc::rc::Rc`] or [`alloc::sync::Arc`] data holder.
-/// Return an sequence of nodes and the anchors.
+/// Return an sequence of nodes and insert the anchors automatically.
 ///
 /// ```
 /// use yaml_peg::{parse, node};
@@ -475,25 +508,22 @@ impl<R: Repr> DerefMut for Loader<'_, R> {
 /// age: 46
 /// ";
 /// // Node with Rc repr
-/// let (root, anchors) = parse(doc).unwrap();
-/// assert_eq!(anchors.len(), 0);
+/// let root = parse(doc).unwrap();
 /// assert_eq!(root, vec![node!({
 ///     "name" => "Bob",
 ///     "married" => true,
 ///     "age" => 46,
 /// })]);
 /// // Node with Arc repr
-/// let (root, anchors) = parse(doc).unwrap();
-/// assert_eq!(anchors.len(), 0);
+/// let root = parse(doc).unwrap();
 /// assert_eq!(root, vec![node!(arc{
 ///     "name" => "Bob",
 ///     "married" => true,
 ///     "age" => 46,
 /// })]);
 /// ```
-pub fn parse<R: Repr>(doc: &str) -> Result<(Seq<R>, Anchor<R>), String> {
-    let mut p = Loader::new(doc.as_bytes());
-    p.parse()
-        .map_err(|e| e.into_error(doc))
-        .map(|a| (a, p.anchors))
+///
+/// Please see [`Loader`] for parser customization.
+pub fn parse<R: Repr>(doc: &str) -> Result<Seq<R>, PError> {
+    Loader::new(doc.as_bytes()).parse()
 }
